@@ -201,15 +201,15 @@ def colorGraph(adjacency):
     """
     # Basic implementation
     # https://en.wikipedia.org/wiki/Greedy_coloring
-    color = dict()
+    colors = dict()
     # Sorted for reproducibility, probably not the optimal vertex order
     for node in sorted(adjacency):
         usedNeighbourColors = {
-            color[neighbour] for neighbour in adjacency[node] if neighbour in color
+            colors[neighbour] for neighbour in adjacency[node] if neighbour in colors
         }
-        color[node] = firstAvailable(usedNeighbourColors)
+        colors[node] = firstAvailable(usedNeighbourColors)
     groups = defaultdict(list)
-    for node, color in color.items():
+    for node, color in colors.items():
         groups[color].append(node)
     return list(groups.values())
 
@@ -559,7 +559,7 @@ class MarkFeatureWriter(BaseFeatureWriter):
                 # We keep the NamedAnchor if the markClass is allowed in the
                 # current lookup.
                 def include(anchor):
-                    return anchor.markClass.name in markClasses
+                    return anchor.markClass.name in markClasses  # noqa: B023
 
                 filteredAttachment = attachment.filter(include)
                 if filteredAttachment:
