@@ -16,6 +16,7 @@ from fontTools.cffLib import (
 from fontTools.misc.arrayTools import unionRect
 from fontTools.misc.fixedTools import otRound
 from fontTools.pens.boundsPen import ControlBoundsPen
+from fontTools.pens.pointPen import SegmentToPointPen
 from fontTools.pens.reverseContourPen import ReverseContourPen
 from fontTools.pens.t2CharStringPen import T2CharStringPen
 from fontTools.pens.ttGlyphPen import TTGlyphPointPen
@@ -99,6 +100,7 @@ class BaseOutlineCompiler:
         glyphOrder=None,
         tables=None,
         notdefGlyph=None,
+        colrLayerReuse=True,
     ):
         self.ufo = font
         # use the previously filtered glyphSet, if any
@@ -114,6 +116,7 @@ class BaseOutlineCompiler:
         self.unicodeToGlyphNameMapping = self.makeUnicodeToGlyphNameMapping()
         if tables is not None:
             self.tables = tables
+        self.colrLayerReuse = colrLayerReuse
         # cached values defined later on
         self._glyphBoundingBoxes = None
         self._fontBoundingBox = None
@@ -971,6 +974,7 @@ class BaseOutlineCompiler:
                 layerInfo,
                 glyphMap=glyphMap,
                 clipBoxes=clipBoxes,
+                allowLayerReuse=self.colrLayerReuse,
             )
 
     def setupTable_CPAL(self):
@@ -1535,6 +1539,7 @@ class StubGlyph:
             self.unicode = None
         if name == ".notdef":
             self.draw = self._drawDefaultNotdef
+            self.drawPoints = self._drawDefaultNotdefPoints
         self.reverseContour = reverseContour
 
     def __len__(self):
@@ -1582,6 +1587,10 @@ class StubGlyph:
         pen.lineTo((xMin, yMin))
         pen.closePath()
 
+    def _drawDefaultNotdefPoints(self, pen):
+        adapterPen = SegmentToPointPen(pen, guessSmooth=False)
+        self.draw(adapterPen)
+
     def _get_controlPointBounds(self):
         pen = ControlBoundsPen(None)
         self.draw(pen)
