@@ -2,9 +2,11 @@ import importlib
 import logging
 import re
 from copy import deepcopy
-from inspect import getfullargspec
+from inspect import currentframe, getfullargspec
+from typing import Set
 
 from fontTools import subset, ttLib, unicodedata
+from fontTools.designspaceLib import DesignSpaceDocument
 from fontTools.feaLib.builder import addOpenTypeFeatures
 from fontTools.misc.fixedTools import otRound
 from fontTools.misc.transform import Identity, Transform
@@ -305,6 +307,18 @@ def unicodeInScripts(uv, scripts):
     return not sx.isdisjoint(scripts)
 
 
+# we consider the 'Common' and 'Inherited' scripts as neutral for
+# determining a script horizontal direction
+DFLT_SCRIPTS = {"Zyyy", "Zinh"}
+
+
+def unicodeScriptDirection(uv):
+    sc = unicodedata.script(chr(uv))
+    if sc in DFLT_SCRIPTS:
+        return None
+    return unicodedata.script_horizontal_direction(sc)
+
+
 def calcCodePageRanges(unicodes):
     """Given a set of Unicode codepoints (integers), calculate the
     corresponding OS/2 CodePage range bits.
@@ -478,8 +492,8 @@ def _loadPluginFromString(spec, moduleName, isValidFunc):
         raise TypeError(klass)
     try:
         options = _kwargsEval(kwargs) if kwargs else {}
-    except SyntaxError:
-        raise ValueError("options have incorrect format: %r" % kwargs)
+    except SyntaxError as e:
+        raise ValueError("options have incorrect format: %r" % kwargs) from e
 
     return klass(**options)
 
@@ -487,3 +501,77 @@ def _loadPluginFromString(spec, moduleName, isValidFunc):
 def quantize(number, factor):
     """Round to a multiple of the given parameter"""
     return factor * otRound(number / factor)
+
+
+def init_kwargs(kwargs, defaults):
+    """Initialise kwargs default values.
+
+    To be used as the first function in top-level `ufo2ft.compile*` functions.
+
+    Raise TypeError with unexpected keyword arguments (missing from 'defaults').
+    """
+    extra_kwargs = set(kwargs).difference(defaults)
+    if extra_kwargs:
+        # get the name of the function that called init_kwargs
+        func_name = currentframe().f_back.f_code.co_name
+        raise TypeError(
+            f"{func_name}() got unexpected keyword arguments: "
+            f"{', '.join(repr(k) for k in extra_kwargs)}"
+        )
+    return {k: (kwargs[k] if k in kwargs else v) for k, v in defaults.items()}
+
+
+def prune_unknown_kwargs(kwargs, *callables):
+    """Inspect callables and return a new dict skipping any unknown arguments.
+
+    To be used after `init_kwargs` to narrow down arguments for underlying code.
+    """
+    known_args = set()
+    for func in callables:
+        known_args.update(getfullargspec(func).args)
+    return {k: v for k, v in kwargs.items() if k in known_args}
+
+
+def ensure_all_sources_have_names(doc: DesignSpaceDocument) -> None:
+    """Change in-place the given document to make sure that all <source> elements
+    have a unique name assigned.
+
+    This may rename sources with a "temp_master.N" name, designspaceLib's default
+    stand-in.
+    """
+    used_names: Set[str] = set()
+    counter = 0
+    for source in doc.sources:
+        while source.name is None or source.name in used_names:
+            source.name = f"temp_master.{counter}"
+            counter += 1
+        used_names.add(source.name)
+
+
+def getMaxComponentDepth(glyph, glyphSet, maxComponentDepth=0):
+    """Return the height of a composite glyph's tree of components.
+
+    This is equal to the depth of its deepest node, where the depth
+    means the number of edges (component references) from the node
+    to the tree's root.
+
+    For glyphs that contain no components, only contours, this is 0.
+    Composite glyphs have max component depth of 1 or greater.
+    """
+    if not glyph.components:
+        return maxComponentDepth
+
+    maxComponentDepth += 1
+
+    initialMaxComponentDepth = maxComponentDepth
+    for component in glyph.components:
+        try:
+            baseGlyph = glyphSet[component.baseGlyph]
+        except KeyError:
+            continue
+        componentDepth = getMaxComponentDepth(
+            baseGlyph, glyphSet, initialMaxComponentDepth
+        )
+        maxComponentDepth = max(maxComponentDepth, componentDepth)
+
+    return maxComponentDepth
